var dfs49000_TransmissaoNfe_Controller = function(General, Dfs49000_TransmissaoNfe_Service, templateManager, ScreenService, MessageService, ZhFilterPreferences, ZHPromise, localStorageEngine) {
    
    var self = this;
    
    var requestError = function(error){
        var errorMessage = "Ocorreu um erro ao tentar realizar a operação.";
        if (error && error.data) {
            errorMessage = error.data.error;
        }
        ScreenService.showMessage(errorMessage, "ERROR");
    };

    var validaCampoEmpresa = function(transmissaoNfeArgs) {
        var empresa = transmissaoNfeArgs.owner.field.widget.getField("NMEMPRESA");
        empresa.readOnly = !transmissaoNfeArgs.row.NMFILIAL;
    };
    
    this.validaCNPJ = function (field) {
        if (!field.value()) {
            return;
        }
        if (!General.validateCNPJ(field.value())) {
            ScreenService.showMessage("CNPJ informado inválido.");
            field.setValue("");
        }
    };

    this.unidadeOnChange = function(transmissaoNfeArgs) {
        validaCampoEmpresa(transmissaoNfeArgs);
        this.empresaUtlizaNFE(transmissaoNfeArgs);
    };
    
    this.controlaCamposProdutorRural = function(args) {
        
        var widget = templateManager.container.getWidget("produtorRural");
        
        var nfe = widget.currentRow.IDMODDOCNFE;
        
        var CPFCNPJ = widget.getField('NRINSJURIEST');
        var inscricaoEstadual = widget.getField('CDINSCESTA');
        var serie = widget.getField('CDSERIE');
        var numero = widget.getField('NRNOTAFISC');
        var dtEmissao = widget.getField('DTEMISSAO');
        var pais = widget.getField('NMPAIS');
        var estado = widget.getField('NMESTADO');
        var chaveAcesso = widget.getField('NRACESSONFE');
        

        if(widget.currentRow.IDMODDOCNFE == "S"){
            CPFCNPJ.readOnly = true;
            inscricaoEstadual.readOnly = true;
            serie.readOnly = true;
            numero.readOnly = true;
            dtEmissao.readOnly = true;
            pais.readOnly = true;
            estado.readOnly = true;
            
            delete CPFCNPJ.validations.required;
            delete inscricaoEstadual.validations.required;
            delete serie.validations.required;
            delete numero.validations.required;
            delete dtEmissao.validations.required;
            delete pais.validations.required;
            delete estado.validations.required;
            
            chaveAcesso.readOnly = false;
            chaveAcesso.validations.required = true;
            
            widget.reload();

        }else{
            CPFCNPJ.readOnly = false;
            inscricaoEstadual.readOnly = false;
            serie.readOnly = false;
            numero.readOnly = false;
            dtEmissao.readOnly = false;
            pais.readOnly = false;
            estado.readOnly = false;
            
            CPFCNPJ.validations.required = true;
            inscricaoEstadual.validations.required = true;
            serie.validations.required = true;
            numero.validations.required = true;
            dtEmissao.validations.required = true;
            pais.validations.required = true;
            estado.validations.required = true;
            
            chaveAcesso.readOnly = true;
            delete chaveAcesso.validations.required;
            
            widget.reload();
        }
    };

    var getTransmissaoNFeWidget = function(){
        if(templateManager.container)
            return templateManager.container.getWidget("transmissaoNFe");
    };

    this.getCurrentField = function(field) {
        if(templateManager.container) {
            if (templateManager.container.getActiveWidget().name === "zhFilterPreferencesSelectConditions" ){
                return !templateManager.container.getActiveWidget().currentRow[field] ? null : templateManager.container.getActiveWidget().currentRow[field];
                
            }else{
                var transmissaoNFeWidget = getTransmissaoNFeWidget();
                if (transmissaoNFeWidget)
                return !transmissaoNFeWidget.currentRow[field] ? null : transmissaoNFeWidget.currentRow[field];
                
            }
        }
        
    };

    this.getWidgetProdutorRural = function(){
        return templateManager.container.getWidget('produtorRural');
    };

    var getFilterPopupCurrentRow = function(args){
        return args.owner.appliedFilter.FILTER_ROW;
    };

    this.getEmpresaFiltrada = function(){
        var transmissaoNFeWidget = getTransmissaoNFeWidget();
        if(transmissaoNFeWidget){
            return transmissaoNFeWidget.appliedFilter.FILTER_ROW.CDEMPRESA;
        }
    };
        
    this.validacoesWidgetOnEnterOnce = function(args) { 
        var widget = args.owner;
        setDefaultFilialFilter(widget).then(defaultFilter =>{
            ZhFilterPreferences.setOriginalFilter(widget.id, defaultFilter);
            ZhFilterPreferences.openFilterWidget(widget, defaultFilter).then (response => {
                var filterWidget = templateManager.container.getActiveWidget();
                filterWidget.actions.filter(action => action.name === "clear")[0].shortcutKey = "ctrl+backspace";
            });
            localStorageEngine.remove("df_saida#/tipo_transmissao");
        });
    };

    var setDefaultFilialFilter = function(widget) {
        var promise = ZHPromise.defer(); 
        var defaultFilter = Util.clone(widget.currentRow);
        defaultFilter.DTENTRSAID = undefined;

        Dfs49000_TransmissaoNfe_Service.buscaOperadorLogado().then(function(operador) {
            operador = operador[0];
            var filialSelecionada = operador.CDFILIALSELECIONADA;
            
            
            // Verifica se há parametro de CDFILIAL de PEDIDO para realizar busca sobre este CDFILIAL
            var filialPed = checkCDFilialOutModule();
            if(filialPed) filialSelecionada = filialPed;
            
            
            var filialField = widget.getField('NMFILIAL');
            var dtEntrField = widget.getField('DTENTRSAID');
            widget.currentRow.CDFILIAL = filialSelecionada;
            setCurrentRowWithOutDataFilial(filialSelecionada, filialField, defaultFilter, dtEntrField).then(callback =>{
                promise.resolve(defaultFilter);
            });
        });
        return promise;
    };
    
    // Função que capta o CDFILIAL Do módulo de Pedido
    var checkCDFilialOutModule = function() {
        
        return Dfs49000_TransmissaoNfe_Service.getLocalVarsFilterOutModule().CDFILIAL;
    };
    
    // Função que retorna Data do módulo de Pedido
    var checkDtEntrSaidOutModule = function() {
        
        return Dfs49000_TransmissaoNfe_Service.getLocalVarsFilterOutModule().DTENTRSAID;
        
    };
    
    // Função que apaga as variaveis de PEDIDO que ficam na cache
    var clearLocalVarsOutModule = function() {
        Dfs49000_TransmissaoNfe_Service.clearLocalVarsOutModule();
    };
    
    var setCurrentRowWithOutDataFilial = function(filialOut, filialField, currentRow, dtEntrField) {
        var promise = ZHPromise.defer();  
        General.callRestEngine('df#/filioper_lancamento_notafiscal', 'FilterData', [{"name": "CDOPERADOR", "operator": "=", "value": "OPERADOR_LOGADO"},
                                                                                    {"name": "CDFILIAL"  , "operator": "=", "value": filialOut}]
        ).then(function(filialLogada) {
            let filialDefaultValue;
            let filialOutData = filialField.outData;
            
            //Utiliza o primeiro dataset retornado. Necessário pois o dataset pode ser filioper_lancamento_notafiscal ou perfacessfili_lancamento_notafiscal
            for (let datasetName in filialLogada.dataset) {
                if (filialLogada.dataset.hasOwnProperty(datasetName)) {
                    filialDefaultValue = filialLogada.dataset[datasetName][0];
            	    break;
                }
            }
            Dfs49000_TransmissaoNfe_Service.buscaDataProcessamento(filialOut).then(([processamento]) => {
                const dtProcessFormat = moment(processamento.DTPROCESSA, 'DD/MM/YYYY').format('DD/MM/YYYY');
                currentRow.DTENTRSAID = {start: dtProcessFormat, period: undefined, end: dtProcessFormat};
               
                // Verifica se há parametro de Data Saida a partir do módulo de Pedidos
                var dtEntrSaid = checkDtEntrSaidOutModule();
                if(dtEntrSaid.start) currentRow.DTENTRSAID = dtEntrSaid;
               
                // Adiciona os outros parmetros advindo do Módulo de Pedidos
                setCurrentRowParamsOutModule(currentRow, Dfs49000_TransmissaoNfe_Service.getLocalVarsFilterOutModule() );
               
                // Limpa dados de Pedido na cache
                clearLocalVarsOutModule();
               
                promise.resolve();
            });
            currentRow.IDENTRSAIDOP = 'S';
            setCurrentRowWithOutData(currentRow, filialDefaultValue, filialOutData);
        });
        return promise;
    };

    // Seta os valores de PEDIDO para o filtro de DF, caso houver falta de algum parametro, os valores não são inseridos
    var setCurrentRowParamsOutModule = function(currentRow, valuesToSetOnCurrentRow) {
        
        var exec = true;
        
        for (var i in valuesToSetOnCurrentRow) {
            
            if(!valuesToSetOnCurrentRow[i]) {
                exec = false;
            }
            
        }
        
        if(exec) {
            
            for (var j in valuesToSetOnCurrentRow) { 
                
                currentRow[j] = valuesToSetOnCurrentRow[j];
    
            }
            
        }
        
    };

    var setCurrentRowWithOutData = function(currentRow, valuesToSetOnCurrentRow, outDataValue) {
        for (var key in outDataValue) {
            currentRow[outDataValue[key]] = valuesToSetOnCurrentRow[key];
        }
    };

    this.getFieldFromFilterPopup = function(field, args){
        var transmissaoNFeWidget = getTransmissaoNFeWidget();
        if(transmissaoNFeWidget){
            if(transmissaoNFeWidget.appliedFilter)
                return transmissaoNFeWidget.appliedFilter.FILTER_ROW[field] || 'TD';
            else
                return 'TD';
        }
    };

    this.criaDataSourceGridFilterAfterApply = function(args) {
        
        args.e.preventDefault();
        
        var transmissaoWidget = args.owner;
        var dataSourceFilter = transmissaoWidget.dataSourceFilter;
        var currentRowPopup = transmissaoWidget.appliedFilter.FILTER_ROW;
        var camposData = ['P_DTENTRSAIDINI', 'P_DTENTRSAIDFIM', 'P_DTEMISSAOINI', 'P_DTEMISSAOFIM'];
        var datasRemover = dataSourceFilter.filter(function(field){
            return camposData.indexOf(field.name) >= 0;
        });
        if(datasRemover && datasRemover.length >= 0){
            datasRemover.forEach(function(data){
                dataSourceFilter.splice(dataSourceFilter.indexOf(data), 1);
            });
        }
        var datasFiltro = dataSourceFilter.filter(function(value) {
            return value.name === "DTENTRSAID" || value.name === 'DTEMISSAO';
        });
        if(datasFiltro.length === 1){
            var dataEmissao = {
                "name": "DTEMISSAO",
                "value": {
                    0: "TD", 1: "TD"
                }
            };
            datasFiltro.push(dataEmissao);
        }
        var filterDefault = {
            'operator': '='
        };
        datasFiltro.forEach(function(data) {
            filterDefault.name = 'P_' + data.name + 'INI';
            filterDefault.value = (data.value[0] != 'TD') ? data.value[0] : '';
            dataSourceFilter.push(Util.clone(filterDefault));
            filterDefault.name = 'P_' + data.name + 'FIM';
            filterDefault.value = (data.value[1] != 'TD') ? data.value[1] : '';
            dataSourceFilter.push(Util.clone(filterDefault));
            if(data.name == 'DTEMISSAO') {
                if(data.value[0] == 'TD') {
                    filterDefault.name = 'P_TDTEMISSAO';
                    filterDefault.value = 'TD';
                    dataSourceFilter.push(Util.clone(filterDefault));
                }
                else {
                    filterDefault.name = 'P_TDTEMISSAO';
                    filterDefault.value = '';
                    dataSourceFilter.push(Util.clone(filterDefault));
                }
            }
            if(dataSourceFilter.indexOf(data) > 0)
                dataSourceFilter.splice(dataSourceFilter.indexOf(data), 1);
        });
        transmissaoWidget.dataSourceFilter = transmissaoWidget.dataSourceFilter.filter(function(field){return field.name[1] == '_' || field.name=="IDSTATUSNFE" ;});
        transmissaoWidget.reload();
    };
    
    this.atualizaFiltro = function(widget) {
      widget.dataSourceFilter[1].operator = '<';
    };

    this.montaDataSourceFilter = function(args){
        args.owner.dataSourceFilter = args.owner.dataSourceFilter.filter(function(field){return field.name[1] == '_';});
    };

    this.preencheDadosCancelamento = function(args){
        args.owner.currentRow = args.owner.getParent().currentRow;
    };

    this.validacoesNotaTransmissaoAfterMoveRow = function(args) {
        this.verificaCamposEpec(args);
        desabilitaActionsEmail(args.owner);
        desabilitaTabProdutorRural(args.owner);
        desabilitaActionCancelarTransmissao(args.owner, args.owner.currentRow.IDSTATUSNFE, args.owner.currentRow.IDSTATUS);
        desabilitaActionCartaCorrecao(args.owner, args.owner.currentRow.IDSTATUSNFE);
        desabilitaActionAlterarDataEmissao(args.owner, args.owner.currentRow.IDSTATUSNFE);
        desabilitaActionVisualizarDanfe(args.owner, args.owner.currentRow.IDSTATUSNFE, args.owner.currentRow.IDTIPOEMISNFE);
        desabilitaActionDownloadXML(args.owner, args.owner.currentRow.IDSTATUSNFE, args.owner.currentRow.IDTIPOEMISNFE);
        let cartaCorrecao = templateManager.container.getWidget('swipeCartaCorrecao');
        cartaCorrecao.showAsTab = false;
        let popupLote = templateManager.container.getWidget('popupAtualizarLote');
        popupLote.showAsTab = false;
    };
    
    this.verificaCamposEpec = function(args){
        var widget = templateManager.container.getWidget('transmissaoNFe');
        if(widget.currentRow.IDTIPOEMISNFE != 0 && widget.currentRow.IDTIPOEMISNFE != 4){
            if(widget.currentRow.DTHRREGCONTNFE == null && widget.currentRow.NRREGDPECNFE == null){
                widget.currentRow.DSOBSSTATUSEPEC = null;
            }    
        }
        widget.reload();
        
    };
    
    this.idorigdestOnChange = function(filterArgs) {
        alteraVisibilidadeCampoInscricaoFiltro(filterArgs);
        limpaCamposOutDataSelectsTpOperacFiltro(filterArgs);
    };

    this.getPaisFiltroEstado = function(){
        var widgetProdutorRural = this.getWidgetProdutorRural();
        return widgetProdutorRural.currentRow.CDPAIS;
    };

    var alteraVisibilidadeCampoInscricaoFiltro = function(filterArgs) {
        var filtroWidget = filterArgs.owner.field.widget;
        var cliente = filtroWidget.getField("NMRAZSOCCLIE");
        var fornecedor = filtroWidget.getField("NMRAZSOCFORN");
        var filial = filtroWidget.getField("DSFILIRECE");
        var tipoDestino = filterArgs.row.IDORIGDESTNF;

        switch (tipoDestino) {
            case "C":
                validacaoCamposOutTpOperacFilter(cliente, fornecedor, filial);
                break;
            case "F":
                validacaoCamposOutTpOperacFilter(fornecedor, cliente, filial);
                break;
            case "L":
                validacaoCamposOutTpOperacFilter(filial, fornecedor, cliente);
                break;
            default:
                filial.readOnly = true;
                cliente.readOnly = true;
                fornecedor.readOnly = true;
        }
    };

    var limpaCamposOutDataSelectsTpOperacFiltro = function(filterArgs)
    {
        var filtroWidget = filterArgs.owner.field.widget;
        var cabecalhoRow = filtroWidget.currentRow;
        var cliente = filtroWidget.getField("NMRAZSOCCLIE");
        var fornecedor = filtroWidget.getField("NMRAZSOCFORN");
        var filial = filtroWidget.getField("DSFILIRECE");

        filial.value(null);

        cliente.value(null);
        cabecalhoRow.CDCLIENTE = null;

        fornecedor.value(null);
        cabecalhoRow.CDFORNECED = null;
        cabecalhoRow.IDSITUCADA = null;
        cabecalhoRow.IDBLQNFRELAFOAF = null;

        cabecalhoRow.NRINSJURNF = null;
        cabecalhoRow.NMINSJURNF = null;
        cabecalhoRow.IDTPINSJURNF = null;
    };

    var validacaoCamposOutTpOperacFilter = function(princialField) {
        princialField.showOnView = true;
        princialField.showOnForm = true;
        princialField.isVisible = true;
        princialField.readOnly = false;

        for (var i = 1; i < arguments.length; i++) {
            arguments[i].showOnView = false;
            arguments[i].showOnForm = false;
            arguments[i].isVisible = false;
            arguments[i].readOnly = true;
        }
    };

    var desabilitaActionsEmail = function(widget){
        var actionEmailNFSelecionada = widget.getAction('emailNFSelecionada');
        var actionEmailTransp = widget.getAction('emailTransp');
        var actionEmailCanc = widget.getAction('emailCanc');
        var actionEmailCancTransp = widget.getAction('emailCancTransp');

        switch (widget.currentRow.IDSTATUSNFE) {
            case "A":
                actionEmailNFSelecionada.isVisible = true;
                if(widget.currentRow.NRINSJURTRAN){
                    actionEmailTransp.isVisible = true;
                }
                else{
                    actionEmailTransp.isVisible = false;
                }
                actionEmailCanc.isVisible = false;
                actionEmailCancTransp.isVisible = false;
                break;
            case "C":
                actionEmailNFSelecionada.isVisible = false;
                actionEmailTransp.isVisible = false;
                actionEmailCanc.isVisible = true;
                if(widget.currentRow.NRINSJURTRAN){
                    actionEmailCancTransp.isVisible = true;
                }
                else{
                    actionEmailCancTransp.isVisible = false;
                }
                break;
            default:
                actionEmailNFSelecionada.isVisible = false;
                actionEmailTransp.isVisible = false;
                actionEmailCanc.isVisible = false;
                actionEmailCancTransp.isVisible = false;
        }
    };

    var desabilitaActionCancelarTransmissao = function(widgetPai, statusNFE, statusNota){
        var widgetCancelamento = templateManager.container.getWidget('popupCancelar');
        var actionCancelarTransmissao = widgetPai.getAction('cancelarTransmissao');
        var actionConfirmarCancelamento = widgetCancelamento.getAction('confirmarCancelamento');

        if(statusNFE === 'A' && statusNota === 'P'){
            actionCancelarTransmissao.isVisible = true;
            actionConfirmarCancelamento.isVisible = true;
            widgetCancelamento.getField('DSMOTIVOCANCNFE').readOnly = false;
        }
        else{
            actionCancelarTransmissao.isVisible = false;
            actionConfirmarCancelamento.isVisible = false;
            widgetCancelamento.getField('DSMOTIVOCANCNFE').readOnly = true;
        }
    };

    var desabilitaActionCartaCorrecao = function(widgetPai, status){
        var actionCartaCorrecao = widgetPai.getAction('novaCartaCorrecao');

        if(status === 'A'){
            actionCartaCorrecao.isVisible = true;
        }
        else{
            actionCartaCorrecao.isVisible = false;
        }
    };

    var desabilitaActionAlterarDataEmissao = function(widgetPai, status){
        var actAlterarDataEmissao = widgetPai.getAction('actAlterarDataEmissao');

        if(status === 'R'){
            actAlterarDataEmissao.isVisible = true;
        }
        else{
            actAlterarDataEmissao.isVisible = false;
        }
    };
    
    var desabilitaActionVisualizarDanfe = function(widgetPai, status, tpEmis){
        let actVisualizarDanfe = widgetPai.getAction('actVisualizarDanfe');

        if(status === 'A' || tpEmis in {4:{}, 5:{}}) { //Libera danfe para nfe aceita, EPEC ou formulário FSDA.
            actVisualizarDanfe.isVisible = true;
        }
        else{
            actVisualizarDanfe.isVisible = false;
        }
    };
    
    var desabilitaActionDownloadXML = function(widgetPai, status, tpEmis){
        let actDownloadXML = widgetPai.getAction('downloadXML');

        if(status in {A:{}, C:{}} || tpEmis == 4) { //Libera xml para nfe cancelada, aceita ou EPEC.
            actDownloadXML.isVisible = true;
        }
        else{
            actDownloadXML.isVisible = false;
        }
    };
    
    var desabilitaTabProdutorRural = function(widget){
        var widgetProdutorRural = widget.container.getWidget('produtorRural');

        if(widget.currentRow.IDENTRSAIDOP === "S"){
            widgetProdutorRural.isVisible = false;
        }
        else{
            switch (widget.currentRow.IDSTATUSNFE) {
                case "P":
                case "R":
                    widgetProdutorRural.isVisible = true;
                    break;
                default:
                    widgetProdutorRural.isVisible = false;
            }
        }
    };

    this.actAlterarDataEmissaoOnClick = function(transmissaoArgs) {
        General.edit(transmissaoArgs.owner.widget);
    };

    this.cancelaEdicaoDataEmissao = function(args){
        General.backOrCancel(args);
    };

    this.alteraDataEmissao = function(args){
        Dfs49000_TransmissaoNfe_Service.alteraDataEmissao(args.row).then(function(callback){
            args.owner.widget.view();
        });
    };

    this.inutNF = function(args){
        var widget = args.owner.widget;
        if(widget.appliedFilter.FILTER_ROW.CDFILIAL){
            sessionStorage.setItem('CDFILIAL',widget.appliedFilter.FILTER_ROW.CDFILIAL);
            sessionStorage.setItem('NMFILIAL',widget.appliedFilter.FILTER_ROW.NMFILIAL);
            sessionStorage.setItem('NRINSJURIEST', widget.appliedFilter.FILTER_ROW.NRINSJURIESTINUTNF);
            sessionStorage.setItem('IDENDEMISFIL', widget.appliedFilter.FILTER_ROW.IDENDEMISFIL);
        }
        if(widget.appliedFilter.FILTER_ROW.CDEMPRESA){
            sessionStorage.setItem('CDEMPRESA',widget.appliedFilter.FILTER_ROW.CDEMPRESA);
            sessionStorage.setItem('NMRAZSOCEMPR',widget.appliedFilter.FILTER_ROW.NMEMPRESA);
        }
        if(widget.appliedFilter.FILTER_ROW.CDINSCESTA)
            sessionStorage.setItem('CDINSCESTA',widget.appliedFilter.FILTER_ROW.CDINSCESTA);
        if(widget.appliedFilter.FILTER_ROW.CDINSCMUNI)
           sessionStorage.setItem('CDINSCMUNI',widget.appliedFilter.FILTER_ROW.CDINSCMUNI);
        ScreenService.openWindow('df_saida#dfs51000_inutilizacao_nota_fiscal_eletronica');
    };

    this.abrirPopupAtualizarLote = function(args){
        var popupAtualizarLote = args.owner.widget.container.getWidget('popupAtualizarLote');
        popupAtualizarLote.editing      = true;
        popupAtualizarLote.showAsTab    = true;
        if(args.owner.name == 'atualizarLote') {
            ScreenService.closePopup();
        }
        ScreenService.openSwipe(popupAtualizarLote).then(() => {
            popupAtualizarLote.reload();
            popupAtualizarLote.activate();
        });
    };

    this.abrirPopupCancelarTransmissao = function(args){
        if(moment().isBefore(moment(args.owner.widget.currentRow.DTEMISSAO, 'DD/MM/YYYY HH:mm:ss').add(1, 'day')))
        {
            var popupCancelar = args.owner.widget.container.getWidget('popupCancelar');
            ScreenService.openPopup(popupCancelar);
        }
        else{
            if(args.owner.widget.appliedFilter.FILTER_ROW.IDCANCEXTMPNFE !== 'S'){
                ScreenService.showMessage('Operação bloqueada! Não é permitido o cancelamento extemporâneo de Nota Fiscal Eletrônica.');
            }
            else{
                var mensagem = "Atenção! Esta Nota Fiscal Eletrônica está sendo cancelada num prazo superior à 24 horas a partir da sua autorização." +
                               " Conforme legislação estadual em vigor esta operação poderá estar sujeito a multa ou não ser permitida." +
                               " Deseja confirmar a operação?";
        		ScreenService.confirmMessage(mensagem, 'question', function (success) {
                    var popupCancelar = args.owner.widget.container.getWidget('popupCancelar');
                    ScreenService.openPopup(popupCancelar);
                });

            }
        }
    };
    
    this.viewNotasLote = function(args) {
        var popupNotaLote = templateManager.container.getWidget('wdg_notaLote');
        popupNotaLote.editing = true;
        popupNotaLote.label = 'Notas Fiscais do Lote ' + args.owner.widget.currentRow.NRLOTENFE;
        ScreenService.openPopup(popupNotaLote).then(function(){
            popupNotaLote.reload();
        });
    };

    this.abrirSwipeCartaCorrecao = function(args){
        General.callRestEngine("df#/verificaTipoOperFili", "Row", args.row).then((success) => {
            if(success.dataset.liberado[0]) {
                var cartaCorrecao = args.owner.widget.container.getWidget('swipeCartaCorrecao');
                cartaCorrecao.showAsTab = true;
                cartaCorrecao.isVisible = true;
                cartaCorrecao.getParent().openTab('swipeCartaCorrecao');
                preencheDadosCartaCorrecao(cartaCorrecao); 
            }
            else {
                ScreenService.showMessage("Não é possivel enviar carta de correção para esta nota. Tipo de operação não permitido para a unidade.");
            }
        });
        
        /*
        ScreenService.openSwipe(swipeCartaCorrecao).then(function(callback){
            preencheDadosCartaCorrecao(swipeCartaCorrecao);
            // swipeCartaCorrecao.view();
        });*/
    };

    this.montaConfig = function(args){
        Dfs49000_TransmissaoNfe_Service.montaConfigTransmissao(getFilterPopupCurrentRow(args)).then(function(resp){
        }, requestError);
    };

    this.verificarStatusServico = function(args) {
        let rows = args.owner.widget.getCheckedRows(),
            notasMarcadas = rows.filter(nota => {
              return nota.IDSTATUSNFE == 'P' || nota.IDSTATUSNFE == 'R';
        });

        if(notasMarcadas.length > 0) {
            Dfs49000_TransmissaoNfe_Service.getTipoTransmissao().then(tipo => {
                if(typeof(tipo) === 'undefined') { //Tipo de transmissão não definido e consulta o status servico.
                    
                    Dfs49000_TransmissaoNfe_Service.statusServico(notasMarcadas).then(status => {
                        if(Object.keys(status[0])[0] === "STATUS") {
                            let popupTransmissao = args.owner.widget.container.getWidget('popupTransmissao'),
                                btnTransNormal = popupTransmissao.getField('transNormal'),
                                btnTransEPEC = popupTransmissao.getField('transEPEC');
                            
                            popupTransmissao.getField("MENSAGEM").label = status[0].STATUS;
                            btnTransEPEC.isVisible = true;
                            
                            if(status[0].aResposta.cStat && status[0].aResposta.cStat != 107) btnTransNormal.isVisible = false;
                            else                                                              btnTransNormal.isVisible = true;
                            
                            if(notasMarcadas.length === 1) {
                                if(notasMarcadas[0].IDTIPOEMISNFE == 4) btnTransEPEC.isVisible = false;
                            }
                            popupTransmissao.reload();
                            ScreenService.openPopup(popupTransmissao);
                        }
                        else if(Object.keys(status[0])[0] === "SUCCESS") {
                            ScreenService.showMessage('Nota(s) transmitida(s) com sucesso!');
                            atualizaStatusNota(notasMarcadas, status[0].SUCCESS);
                            args.owner.widget.unSelectAll();
                            args.owner.widget.reload();
                        }
                        else if(Object.keys(status[0])[0] === "ERRORS") {
                            if(typeof status[0].ERRORS === 'string')
                                ScreenService.showMessage(status[0].ERRORS, "ERROR");
                            else{
                                let popupErros = args.owner.widget.container.getWidget('popupErros'),
                                    erros = [];
                                
                                self.showFieldsPopupErros(popupErros.fields, false);
                                for(let i in status[0].ERRORS) {
                                    erros[i] = { DSERRO: status[0].ERRORS[i] };
                                }
                                popupErros.dataSource.data = erros;
                                popupErros.reload();
                                ScreenService.openPopup(popupErros);
                            }
                        }
                        else{
                            ScreenService.showMessage(status[0].STATUS);
                        }
                    });
                }
                else { //Tipo de transmissão já definido.
                    this.transmiteNotasSelecionadas(tipo);
                }
            });
        }
        else {
            ScreenService.showMessage("Todas as notas selecionadas já foram transmitidas.");
        }
    };
    
    this.saveExecTransmis = (type) => { //type = '' é transmissão normal.
        let confirma = false,
            wdgTransmissao = templateManager.container.getWidget('transmissaoNFe'),
            actClean = wdgTransmissao.getAction('cleTypeTransmis'),
            msgPopupTransmissao = templateManager.container.getActiveWidget().getField('MENSAGEM').label,
            envia = msgPopupTransmissao !== 'Selecionar Modalidade de Transmissão/Emissão';
            
        if(envia) {
            MessageService.confirmMessageByCode('TIPO_TRANSMISSAO')
            .then(() => {
                confirma = true;
                actClean.isVisible = true;
                Dfs49000_TransmissaoNfe_Service.setTipoTransmissao(type);
            })
            .finally(() => {
                if(!confirma) {
                    localStorageEngine.remove("df_saida#/tipo_transmissao");
                    actClean.isVisible = false;
                }
                this.transmiteNotasSelecionadas(type);
            });
        }
        else { //Selecionar modalidade de transmissão.
            Dfs49000_TransmissaoNfe_Service.setTipoTransmissao(type);
            actClean.isVisible = true;
            ScreenService.closePopup(true);
            wdgTransmissao.reload();
            wdgTransmissao.activate();
            MessageService.successNotificationByCode('TIPO_TRANSMIS_DEFINIDO');
        }
    };
    
    this.cleanTypeTransmis = (args) => {
        let actClean = args.owner.widget.getAction('cleTypeTransmis'),
            wdgTransmissao = args.owner.widget;
        
        localStorageEngine.remove("df_saida#/tipo_transmissao");
        actClean.isVisible = false;
        wdgTransmissao.reload();
        ScreenService.closePopup();
        MessageService.successNotificationByCode('TIPO_TRANSMIS_RESET');
    };
    
    this.selectTypeTransmis = (popupTransmissao) => {
        let btnTransNormal = popupTransmissao.getField('transNormal'),
            btnTransEPEC   = popupTransmissao.getField('transEPEC'),
            msg            = popupTransmissao.getField("MENSAGEM");
        
        btnTransNormal.isVisible = true;
        btnTransEPEC.isVisible   = true;
        msg.label = 'Selecionar Modalidade de Transmissão/Emissão';
        
        popupTransmissao.reload();
        ScreenService.openPopup(popupTransmissao);
    };
    
    this.recarregarCampo = function(nomeWidget,nomeCampo){
        var widget = templateManager.container.getWidget(nomeWidget);
        widget.getField(nomeCampo).reload();
    };

    this.transmiteNotasSelecionadas = function(tipo) {
        let wdgTransmissao = templateManager.container.getWidget('transmissaoNFe'),
            rows = wdgTransmissao.getCheckedRows(),
            transmite = true,
            notasMarcadas = rows.filter(nota => {
              return nota.IDSTATUSNFE == 'P' || nota.IDSTATUSNFE == 'R';
        });
        
        if(notasMarcadas.length > 0) {
            if(tipo == 'EPEC') {
               if(notasMarcadas.length > 1) {
                   MessageService.showMessageByCode('BLOCK_TRANSMISSAO_EPEC');
                   transmite = false;
               }
               else if(notasMarcadas[0].IDTIPOEMISNFE == 4) {
                   MessageService.showMessageByCode('REPEAT_TRANSMISSAO_EPEC');
                   transmite = false;
               }
            }
            if(transmite) {
                Dfs49000_TransmissaoNfe_Service.transmiteNotas(notasMarcadas, tipo).then(function(transmissao){
                    ScreenService.closePopup(true);
                    //ScreenService.goBack();
                    
                    if(Object.keys(transmissao[0])[0] === "SUCCESS"){
                        if(tipo != 'FSDA') ScreenService.showMessage('Nota(s) transmitida(s) com sucesso!');
                        else               MessageService.showMessageByCode('TRANSMISSAO_FSDA');
                        
                        atualizaStatusNota(notasMarcadas, transmissao[0].SUCCESS);
                        wdgTransmissao.unSelectAll();
                        wdgTransmissao.reload();
                    }
                    else if(Object.keys(transmissao[0])[0] === "ERRORS"){
                        if(typeof transmissao[0].ERRORS === 'string')
                            ScreenService.showMessage(transmissao[0].ERRORS, "ERROR");
                        else{
                            let popupErros = templateManager.container.getWidget('popupErros'),
                                erros = []; 
                            
                            self.showFieldsPopupErros(popupErros.fields, false);
                            for(let i in transmissao[0].ERRORS) {
                                erros[i] = { DSERRO: transmissao[0].ERRORS[i] };
                            }
                            popupErros.dataSource.data = erros;
                            popupErros.reload();
                            ScreenService.openPopup(popupErros);
                        }
                    }
                    else{
                        ScreenService.showMessage(transmissao[0]);
                    }
                });
            }
        }
        else {
            ScreenService.showMessage("Todas as notas selecionadas já foram transmitidas.");
        }
    };
    
    this.atualizaLotesSelecionados = function(args){
        let lotesMarcados = args.owner.widget.getCheckedRows(),
            lotesMarcadosEnvio     = lotesMarcados.map((row) => {
            return Util.clone(row, ["__rightActions", "__leftActions"]);
        });
        
        Dfs49000_TransmissaoNfe_Service.atualizaLote(lotesMarcadosEnvio).then(function(respLote) {
            if(Object.keys(respLote[0])[0] === "SUCCESS") {
                ScreenService.showMessage('Lote(s) atualizado(s) com sucesso!');
                args.owner.widget.unSelectAll();
                templateManager.container.getWidget('transmissaoNFe').activate(); 
                ScreenService.closeSwipe();
                args.owner.widget.getParent().reload();
                args.owner.widget.getParent().activate();
                
                if(respLote[0].NOTAS_SUCCESS.length) {
                    Dfs49000_TransmissaoNfe_Service.getConfig().then((config) => {
                        if(config.aprovacaoAutomatica == 'S') {
                            Dfs49000_TransmissaoNfe_Service.aprovarNotasTransmissao(respLote[0].NOTAS_SUCCESS).then((successAprovacao) => {
                                if(Object.keys(successAprovacao).length == 1 ) {
                                    if(Object.keys(successAprovacao[0])[0] === 'SUCESSO_APROVACAO' ) {
                                        MessageService.successNotificationByCode('SUCESSO_APROVACAO_LOTE');
                                    }
                                } else if(successAprovacao.aprovarNotasTransmissao) {
                                    if(Object.keys(successAprovacao.aprovarNotasTransmissao)[0] === 'ERROS_APROVACAO' ) {
                                        let erros   = successAprovacao.aprovarNotasTransmissao.ERROS_APROVACAO,
                                            dsErros = Dfs49000_TransmissaoNfe_Service.errosNotasTransmissao(erros);
                                        
                                        self.showPopupErros(dsErros);
                                    } else {
                                        MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                        console.error(successAprovacao);
                                    }
                                } else {
                                    MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                    console.error(successAprovacao);
                                }
                            }, (error) => {
                                MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                console.error(error);
                            });
                        }
                        if(config.emailAutomatico == 'S') {
                            Dfs49000_TransmissaoNfe_Service.emailAutomaticoTransmissao(respLote[0].NOTAS_SUCCESS, config).then((successEmail) => {
                                if(Object.keys(successEmail[0])[0] === 'SUCESSO_EMAIL' ) {
                                        MessageService.successNotificationByCode('SUCESSO_EMAIL_LOTE');
                                } else if(Object.keys(successEmail[0])[0] === 'ERROS_EMAIL' ) {
                                    let erros   = successEmail[0].ERROS_EMAIL,
                                        dsErros = Dfs49000_TransmissaoNfe_Service.errosNotasTransmissao(erros);
                                    
                                    self.showPopupErros(dsErros);
                                }
                                else {
                                    MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                    console.error(successEmail);
                                }
                            }, (error) => {
                                MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                console.error(error);
                            });
                        }
                    });
                }
            }
            else if(Object.keys(respLote[0])[0] === "ERRORS") {
                args.owner.widget.unSelectAll();
                ScreenService.showMessage(respLote[0].ERRORS, "ERROR");
            }
            else {
                args.owner.widget.unSelectAll();
                ScreenService.successNotification(respLote[0]);
            }
        }, (error) => {
            ScreenService.errorNotification("Ocorreu um erro ao atualizar o lote. Tente novamente mais tarde.");
            console.error(error);
        });
    };
    
    this.showPopupErros = (dsErros) => {
        let popupErros = templateManager.container.getWidget('popupErros'),
            atualWidget = templateManager.container.getActiveWidget();
        
        if(popupErros.name != atualWidget.name) {
            this.showFieldsPopupErros(popupErros.fields, true);
            popupErros.dataSource.data = dsErros;
            ScreenService.openPopup(popupErros);
            MessageService.errorNotificationByCode('ERROR_APROVACAOEMAIL_LOTE');
        } else {
            let newDsErros = popupErros.dataSource.data.concat(dsErros);
            popupErros.dataSource.data = newDsErros;
        }
        popupErros.reload();
    };
    
    this.loadRejeicaoXML = function(args) {
        var rejeicaoXML = args.owner.widget.container.getWidget('WidgetRejeicaoXML');
        var rejeicaoXMLTab = args.owner.widget.container.getWidget('WidgetRejeicaoXMLTabbed');
        var DSARQLOTEXMLTab = args.owner.widget.container.getWidget('WIDDSARQLOTEXML_CLOB').getField('DSARQLOTEXML_CLOB');
        var DSARQLOTEXML_CLOB_FORMATTab = args.owner.widget.container.getWidget('WIDDSARQLOTEXML_CLOB_FORMAT').getField('DSARQLOTEXML_CLOB_FORMAT');
        General.callRestEngine("df#/verificaRejeicaoXML", "Row", args.owner.widget.currentRow).then((success) => {
            if(success.dataset.falhaXML[0]) {
                let falhaXML = success.dataset.falhaXML;
                DSARQLOTEXMLTab.setValue(falhaXML[0].DSARQLOTEXML_CLOB);
                DSARQLOTEXML_CLOB_FORMATTab.setValue(falhaXML[0].DSARQLOTEXML_CLOB_FORMAT);
            }   
            else {
                //ScreenService.closePopup(rejeicaoXML);
                ScreenService.showMessage("XML não existe");
                ScreenService.closePopup(rejeicaoXML);
                ScreenService.closePopup(rejeicaoXMLTab);
                rejeicaoXML.isVisible = false;
                rejeicaoXMLTab.isVisible = false;
            }
        });
    };
    
    this.copiaRejeicaoXML = function(args) {
        document.getElementById('DSARQLOTEXML_CLOB').select();
        setTimeout(document.execCommand('copy'),500);
    };
    this.openPopupRejeicaoXML = function(args) {
        if(args.currentRow.IDSTATUSNFE == "R") {
            var wdgRejXML = templateManager.container.getWidget('WidgetRejeicaoXML');
            var wdgRejXMLTab = templateManager.container.getWidget('WidgetRejeicaoXMLTabbed');
                ScreenService.openPopup(wdgRejXML);
                ScreenService.openPopup(wdgRejXMLTab);
        }
        else {
            ScreenService.showMessage('Nota transmitida não está rejeitada.', 'ERROR');
        }
        
    };
    
    this.closePopupRejeicaoXML = function() {
        var wdgRejXML = templateManager.container.getWidget('WidgetRejeicaoXML');
        var wdgRejXMLTab = templateManager.container.getWidget('WidgetRejeicaoXMLTabbed');
        ScreenService.closePopup(wdgRejXML);
        ScreenService.closePopup(wdgRejXMLTab);
        wdgRejXML.isVisible = false;
        wdgRejXMLTab.isVisible = false;
    };
    
    this.goBackPopupErros = (args) => {
        args.owner.widget.dataSource.data = [];
        args.owner.widget.container.getWidget('transmissaoNFe').activate();
        ScreenService.closePopup();
    };
    
    this.showFieldsPopupErros = (fields, exibe) => {
        fields.forEach(field => {
            if(field.name != 'DSERRO') field.isVisible = exibe;
        });
    };

    this.cancelarTransmissao = function(args){
        var motivoCancelamento = args.owner.widget.currentRow.DSMOTIVOCANCNFE;
        if(!motivoCancelamento || motivoCancelamento.trim().length < 15)
            ScreenService.showMessage('Motivo deve conter no mínimo 15 caracteres', 'ERROR');
        else{
            Dfs49000_TransmissaoNfe_Service.cancelarTransmissao(args.row).then((resp) => {
                if(Object.keys(resp[0])[0] === "SUCCESS"){
                    args.owner.widget.currentRow.IDSTATUSNFE = 'C';
                    args.owner.widget.currentRow.DTHRPROTOCOLNFE = resp[0].SUCCESS.DTHRPROTOCOLNFE;
                    args.owner.widget.currentRow.DSOBSSTATUSNFE = resp[0].SUCCESS.DSOBSSTATUSNFE;
                    ScreenService.successNotification("Nota cancelada com sucesso.");
                    ScreenService.closePopup(args.owner.widget);
                    ScreenService.goBack();
                    args.owner.widget.getParent().reload();
                    
                    Dfs49000_TransmissaoNfe_Service.getConfig().then(function(config){
                        if(config.emailAutomatico === 'S'){
                            args.owner.widget.currentRow.CDFILIAL = resp[0].SUCCESS.CDFILIAL;
                            args.owner.widget.currentRow.NRLANCTONF = resp[0].SUCCESS.NRLANCTONF;
                            Dfs49000_TransmissaoNfe_Service.enviarEmailNF(args.owner.widget.currentRow, 'C').then(function(resp){
                                if(Object.keys(resp[0])[0] === "SUCCESS"){ 
                                    args.owner.widget.currentRow.IDEMAILNFE = 'E';
                                }
                                else if(Object.keys(resp[0])[0] === "ERRORS"){
                                    MessageService.showMessageByCode('EMAIL_NAO_ENVIADO_NF');
                                }
                            });
                        }
                    });
                }
                else if(Object.keys(resp[0])[0] === "ERRORS"){
                    ScreenService.showMessage(resp[0].ERRORS, "ERROR");
                }
                else{
                    ScreenService.showMessage(resp[0]);
                }
            });
        }
    };

    this.visualizarDanfe = function(args){
        let nota = args.owner.widget.currentRow;
        
        if(nota.IDSTATUSNFE == 'A' || nota.IDTIPOEMISNFE in {4:{}, 5:{}}) { //Libera danfe para nfe aceita, EPEC ou formulário FSDA.
            Dfs49000_TransmissaoNfe_Service.visualizarDanfe([nota]).then(function(resp){
                resp.forEach(function(fileUri) {
                    this.downloadArquivo(fileUri);
                }.bind(this));
            }.bind(this), function(error) {
                ScreenService.showMessage(error.data.error);
            }.bind(this));
        }
        else{
            ScreenService.showMessage("Esta nota não foi aceita.");
        }
    };
    
    this.downloadArquivo = function(nmArquivo){
        window.open(window.serviceUrls.filter(function(x){return x.package == 'df';})[0].baseUrl+"/downloadArquivo?requestType=Row&row[DSARQUIVO]=" + nmArquivo);
    };

    // this.visualizarNota = function(args){
    //     var filter = [{
    //                 "name": "NRNOTAFISC",
    //                 "operator": "=",
    //                 "value": "000000034"
    //             },
    //             {
    //                 "name": "IDENTRSAIDOP",
    //                 "operator": "=",
    //                 "value": "S"
    //             },
    //             {
    //                 "name": "CDFILIEMIT",
    //                 "operator": "=",
    //                 "value": "0001"
    //             },
    //             {
    //                 "name": "DTENTRSAID",
    //                 "operator": "BETWEEN",
    //                 "value": ["05/09/2016", "09/09/2016"]
    //             }];
    //     ScreenService.openWindow('df_saida#dfs11000_lancamento_saida', filter);
    // }

    this.emissaoBoleto = function(widget){
        sessionStorage.setItem('WIDGET_NAME', widget.name);
        sessionStorage.setItem('CDEMPRESA', widget.currentRow.CDEMPRESA);
        sessionStorage.setItem('NMEMPRESA', widget.currentRow.NMEMPRESA);
        sessionStorage.setItem('NRNOTAFISC', widget.currentRow.NRNOTAFISC);
		
        this.openWindow("fin_contaReceber#rec00001_titulos_a_receber");
    };
    
    this.openWindow = function(windowName, newWindow) {
        Dfs49000_TransmissaoNfe_Service.requestModulesMetadata().then(function (modules) {
            let subModule = windowName.split('#')[0];
            let module = subModule.split('_')[0];
            let url = window.location.origin + '/' + modules[module].baseUrl + "/#/" + windowName;
            ScreenService.openLink(url, false);
        });
    };

    this.downloadDanfeNotasSelecionadas = function(args){
        let notasMarcadas = args.owner.widget.getCheckedRows(),
            notas = notasMarcadas.filter(notafiscal => {
            return notafiscal.IDSTATUSNFE === "A"|| notafiscal.IDTIPOEMISNFE in {4:{}, 5:{}}; //Libera danfe para nfe aceita, EPEC ou formulário FSDA.
        });
        
        if(notas.length > 0){
            Dfs49000_TransmissaoNfe_Service.downloadDanfe(notas).then(fileUri => {
                this.downloadArquivo(fileUri[0]);
            }, error => {
                ScreenService.showMessage(error.data.error);
            });
        }
        else{
            ScreenService.showMessage("Nenhuma nota aceita foi marcada");
        }
    };

    this.downloadXMLNotasSelecionadas = function(args){
        let row = args.owner.widget.currentRow;
        
        if(row.IDSTATUSNFE in {A:{}, C:{}} || row.IDTIPOEMISNFE == 4) { //Libera xml para nfe cancelada, aceita ou EPEC.
            Dfs49000_TransmissaoNfe_Service.downloadXML(row).then(fileUri => {
                this.downloadArquivo(fileUri[0]);
            }, error => {
                console.error(error);
                
                if(typeof(error.data.error) === 'string') ScreenService.showMessage(error.data.error);
                else                                      MessageService.showMessageByCode('ERRO_GERAL');
            });
        }
        else {
            MessageService.showMessageByCode('BLOCK_DOWN_XML');
        }
    };

    this.enviarEmailPendenteNFsAceitas = function(args){
        var notasFiltradas= args.dataSource.data;
        var notas = notasFiltradas.filter(function(notafiscal){
            return notafiscal.IDSTATUSNFE === "A" && notafiscal.IDEMAILNFE === "P";
        });
        if(notas.length > 0){
            Dfs49000_TransmissaoNfe_Service.enviarEmailPendenteNFsAceitas(notas).then(function(resp){
                if(Object.keys(resp[0])[0] === "SUCCESS"){
                    ScreenService.closePopup(args.owner.widget);
                    args.owner.widget.activate();
                    args.owner.widget.currentRow.IDEMAILNFE = 'E';
                    ScreenService.showMessage(resp[0].SUCCESS);
                }
                else if(Object.keys(resp[0])[0] === "ERRORS"){
                    ScreenService.showMessage(resp[0].ERRORS, "ERROR");
                }
                else{
                    ScreenService.showMessage(resp[0]);
                }
            });
        }
        else{
            ScreenService.showMessage("Nenhuma nota aceita com e-mail pendente");
        }
    };

    this.enviarEmailNF = function(args, tipo){
        Dfs49000_TransmissaoNfe_Service.enviarEmailNF(args.owner.widget.currentRow, tipo).then(function(resp){
            if(Object.keys(resp[0])[0] === "SUCCESS"){
                ScreenService.closePopup(args.owner.widget);
                args.owner.widget.activate();
                args.owner.widget.currentRow.IDEMAILNFE = 'E';
                ScreenService.showMessage(resp[0].SUCCESS);
            }
            else if(Object.keys(resp[0])[0] === "ERRORS"){
                ScreenService.showMessage(resp[0].ERRORS, "ERROR");
            }
            else{
                ScreenService.showMessage(resp[0]);
            }
        });
    };

    this.dataAtualMenos15segundos = function(){
        return moment().add(1, 'hours').format("DD/MM/YYYY HH:mm:ss");
    };

    var atualizaStatusNota = function(notasMarcadas, transmissao){
        notasMarcadas.forEach(function(nota){
            nota.DTEMISSAO = this[nota.NRLANCTONF].DTEMISSAO;
            nota.NRACESSONFE = this[nota.NRLANCTONF].NRACESSONFE;
            nota.NRNOTAFISC = this[nota.NRLANCTONF].NRNOTAFISC;
            nota.NRNOTAFISCALELE = this[nota.NRLANCTONF].NRNOTAFISCALELE;
            nota.NRPROTOCOLONFE = this[nota.NRLANCTONF].NRPROTOCOLONFE;
            nota.IDSTATUSNFE = this[nota.NRLANCTONF].IDSTATUSNFE;
        }, transmissao);
    };

    this.salvaProdutorRural = function(args) {
        var widgetProdutorRural = args.owner.widget;
        if (widgetProdutorRural.isValid()) {
            var rowTransmissao = widgetProdutorRural.getParent().currentRow;
            args.row.CDFILIAL = rowTransmissao.CDFILIAL;
            args.row.NRLANCTONF = rowTransmissao.NRLANCTONF;
            args.row.NRACESSONFE = widgetProdutorRural.currentRow.NRACESSONFE.split('.').join("");
            if(args.row.__is_new)
                args.row.NRSEQPRODRURAL = Dfs49000_TransmissaoNfe_Service.getProxSeqProdRural(widgetProdutorRural.dataSource.data);
            if(args.row.NRINSJURIEST.length === 11)
                args.row.IDTPINSCJUR = 'F';
            else if(args.row.NRINSJURIEST.length === 14)
                args.row.IDTPINSCJUR = 'J';
            else if(args.row.NRINSJURIEST.length === 0)
                args.row.IDTPINSCJUR = null;
            else
                return requestError({data: {error:'Inscrição informada inválida'}});
            args.row.NRLANCTONF = rowTransmissao.NRLANCTONF;
            Dfs49000_TransmissaoNfe_Service.salvaProdutorRural(args.row).then(function(callback){
                widgetProdutorRural.currentRow.__is_new = false;
                widgetProdutorRural.view();
            }, requestError);
        }
    };

    this.editarProdRural = function(args){
        General.edit(args.owner.widget);
    };

    this.excluirProdutorRural = function(args) {
        Dfs49000_TransmissaoNfe_Service.excluirProdutorRural(args.row).then(function(callback){
            General.backOrCancel(args);
            var rowExcluida = args.owner.widget.dataSource.data.filter(function(prodRural){
                return args.row.NRLANCTONF === prodRural.NRLANCTONF;
            });
            var index = args.owner.widget.dataSource.data.indexOf(rowExcluida[0]);
            args.owner.widget.dataSource.data.splice(index, 1);
            ScreenService.showMessage('Produtor Rural excluido com sucesso');
        }, requestError);
    };

    this.novoProdRural = function(args){
        args.owner.widget.newRow();
    };

    // var preencheDadosCce = true;

    var preencheDadosCartaCorrecao = function(widgetCartaCorrecao){
        widgetCartaCorrecao.list();
        widgetCartaCorrecao.reload().then(function(callback){
            widgetCartaCorrecao.currentRow = widgetCartaCorrecao.dataSource.data[0] || [];
            widgetCartaCorrecao.view();
        });
    };

    var padNrseqcce = function(nrseqcce){
        var str = "" + nrseqcce;
        var pad = "00";
        return pad.substring(0, pad.length - str.length) + str;
    };

    this.editarCartaCorrecao = function(args){
        var widgetCartaCorrecao = args.owner.widget;
        widgetCartaCorrecao.currentRow.NRSEQCCE = padNrseqcce(parseInt(widgetCartaCorrecao.currentRow.NRSEQCCE || 0) + 1);
        widgetCartaCorrecao.currentRow.NRPROTOCOLCCE = "";
        widgetCartaCorrecao.currentRow.DTHRPROTOCOLCCE = "";
        widgetCartaCorrecao.currentRow.DSOBSSTATUSCCE = "";
        widgetCartaCorrecao.currentRow.DSCORRECAONFE    = "";
        widgetCartaCorrecao.template = 'widget/form.html';
        widgetCartaCorrecao.templateToForm = 'widget/form.html';
        widgetCartaCorrecao.edit();
        widgetCartaCorrecao.isVisible = true;
    };

    this.actCancelarEdicaoCce = function(args){
        var widgetCartaCorrecao = args.owner.widget;
        General.backOrCancel(args);
		widgetCartaCorrecao.showAsTab = false;
        widgetCartaCorrecao.isVisible = false;
        widgetCartaCorrecao.currentRow.NRSEQCCE = widgetCartaCorrecao.currentRow.NRSEQCCE - 1; 
        widgetCartaCorrecao.template = 'widget/form-view.html';
        widgetCartaCorrecao.templateToForm = 'widget/form-view.html';
        ScreenService.closeSwipe();
        templateManager.container.getWidget('transmissaoNFe').activate();
    };
    
    this.actVoltarCce = function(args){
        var widgetCartaCorrecao = args.owner.widget;
		widgetCartaCorrecao.showAsTab = false;
        widgetCartaCorrecao.isVisible = false;
    };

    this.confirmarCartaCorrecao = function(args){
		var widgetCartaCorrecao = args.owner.widget;
		if(!widgetCartaCorrecao.currentRow.DSCORRECAONFE || (widgetCartaCorrecao.currentRow.DSCORRECAONFE.length < 15 || widgetCartaCorrecao.currentRow.DSCORRECAONFE.length > 1000))
		    return ScreenService.showMessage("Descrição da correção deve ter no mínimo 15 e no máximo 1000 caracteres");

		var mensagem = "A Carta de Correção é disciplinada pelo § 1º-A do art. 7º do Convênio S/N," +
                           " de 15 de dezembro de 1970 e pode ser utilizada para regularização de erro ocorrido na emissão de documento fiscal," +
                           " desde que o erro não esteja relacionado com:" +
                           " I - as variáveis que determinam o valor do imposto tais como: base de cálculo, alíquota, diferença de preço, quantidade, valor da operação ou da prestação;" +
                           " II - a correção de dados cadastrais que implique mudança do remetente ou do destinatário;" +
                           " III - a data de emissão ou de saída." +
                           " Confirma a Condição de Uso da Carta de Correção?";
		ScreenService.confirmMessage(mensagem, 'question', function (success) {
            Dfs49000_TransmissaoNfe_Service.enviarCartaCorrecao(getTransmissaoNFeWidget().currentRow, widgetCartaCorrecao.currentRow.DSCORRECAONFE, widgetCartaCorrecao.currentRow.NRSEQCCE).then(function(resp){
                if(resp) {
                    if(Object.keys(resp[0])[0] === "SUCCESS") {
                        ScreenService.successNotification("Carta de Correção enviada com sucesso.");
                        Dfs49000_TransmissaoNfe_Service.getConfig().then(function(config){
                            if(config.emailAutomatico === 'S'){
                                Dfs49000_TransmissaoNfe_Service.enviarEmailCC(getTransmissaoNFeWidget().currentRow, widgetCartaCorrecao.currentRow.NRSEQCCE, "CCe").then(function(resp){
                                    if(Object.keys(resp[0])[0] === "ERRORS") {
                                        MessageService.showMessageByCode('EMAIL_NAO_ENVIADO_CC');
                                    }
                                });
                            }
                        });
                    } else {
                        ScreenService.showMessage(resp[0].ERRORS);
                    }
                } else {
                    MessageService.showMessageByCode('ERROR_ENVIOCCE');
                    console.error(resp);
                }
                preencheDadosCartaCorrecao(widgetCartaCorrecao);
                widgetCartaCorrecao.template = 'widget/form-view.html';
                widgetCartaCorrecao.templateToForm = 'widget/form-view.html';
                widgetCartaCorrecao.reload();
            }, function(error) {
                MessageService.showMessageByCode('ERROR_ENVIOCCE');
                console.error(error);
            });
        });
    };
    
    this.reenvioMailCartaCorrecao = (widgetCartaCorrecao) => {
        MessageService.confirmMessageByCode('REENVIO_MAILCCE').then(confirm => {
            Dfs49000_TransmissaoNfe_Service.enviarEmailCC(getTransmissaoNFeWidget().currentRow, widgetCartaCorrecao.currentRow.NRSEQCCE, "CCe").then(function(resp){
                if(Object.keys(resp[0])[0] === "SUCCESS") {
                    ScreenService.successNotification(resp[0].SUCCESS);
                }
                else if(Object.keys(resp[0])[0] === "ERRORS") {
                    if(typeof resp[0].ERRORS === 'string') ScreenService.showMessage(resp[0].ERRORS, "ERROR");
                    else MessageService.showMessageByCode('ERROR_ENVIOCCE');
                }
                else MessageService.showMessageByCode('ERROR_ENVIOCCE');
            }, (error) => {
                MessageService.showMessageByCode('ERROR_ENVIOCCE');
                console.error(error);
            });
        });
    };
    
    this.cartaCorrecaoLoad = (widgetCartaCorrecao) => {
        let datasource = widgetCartaCorrecao.dataSource.data;
        
        if(datasource.length) widgetCartaCorrecao.getAction('reenvioMailCartaCorrecao').isVisible = true;
        else                  widgetCartaCorrecao.getAction('reenvioMailCartaCorrecao').isVisible = false;
    };
    
    this.popupAtualizarLoteClose = function(widget) {
        ScreenService.closeSwipe();  
        widget.showAsTab = false;
        templateManager.container.getWidget('transmissaoNFe').activate();
    };
    
    this.consultaSituacaoNF = function(widget) {
        if(widget.currentRow.IDSTATUSNFE != 'R' && widget.currentRow.IDSTATUSNFE != 'T') {
            MessageService.showMessageByCode('BLOCK_CONSULTA_SITUACAONF');
        }
        else if(widget.currentRow.IDSTATUSNFE == 'T') {
            General.callRestEngine('df#/crud/nota_lote', 'FilterData', [{"name": "P_CDEMPLOTENFE", "operator": "=", "value": widget.currentRow.CDEMPRESA},
                                                                        {"name": "CDFILIAL"      , "operator": "=", "value": widget.currentRow.CDFILIAL},
                                                                        {"name": "NRLANCTONF"    , "operator": "=", "value": widget.currentRow.NRLANCTONF},
                                                                        {"name": "NRORG"         , "operator": "=", "value": widget.currentRow.NRORG}]
            ).then((result) => {
                let nota = result.dataset.nota_lote;
                
                if(nota.length) {
                    if(!nota[0].NRRECIBONFE) {
                        self.realizaConsultaSituacaoNF(widget);
                    } else {
                        MessageService.showMessageByCode('BLOCK_CONSULTA_SITUACAONF');
                    }
                } else {
                    MessageService.showMessageByCode('BLOCK_CONSULTA_SITUACAONF');
                }
            });
        } else {
            this.realizaConsultaSituacaoNF(widget);
        }
    }; 
    
    this.realizaConsultaSituacaoNF = (widget) => {
        Dfs49000_TransmissaoNfe_Service.getConfig().then((config) => {
            Dfs49000_TransmissaoNfe_Service.consultaSituacaoNF(widget.currentRow, config).then((respConsulta) => {
                ScreenService.goBack();
                ScreenService.closePopup();
                widget.reload();
                
                if(Object.keys(respConsulta[0])[0] === 'SUCCESS') {
                    MessageService.successNotificationByCode("CONSULTA_SITUACAONF");
                    
                    if(Object.keys(respConsulta[0])[1] === 'NOTAS_SUCCESS') {
                        if(respConsulta[0].NOTAS_SUCCESS.length) {
                            if(config.aprovacaoAutomatica == 'S') {
                                Dfs49000_TransmissaoNfe_Service.aprovarNotasTransmissao(respConsulta[0].NOTAS_SUCCESS).then((successAprovacao) => {
                                    if(Object.keys(successAprovacao).length == 1 ) {
                                        if(Object.keys(successAprovacao[0])[0] === 'SUCESSO_APROVACAO' ) {
                                            MessageService.successNotificationByCode('SUCESSO_APROVACAO_LOTE');
                                        }
                                    } else if(successAprovacao.aprovarNotasTransmissao) {
                                        if(Object.keys(successAprovacao.aprovarNotasTransmissao)[0] === 'ERROS_APROVACAO') {
                                            let erros   = successAprovacao.aprovarNotasTransmissao.ERROS_APROVACAO,
                                                dsErros = Dfs49000_TransmissaoNfe_Service.errosNotasTransmissao(erros);
                                            
                                            self.showPopupErros(dsErros);
                                        } else {
                                            MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                            console.error(successAprovacao);
                                        }
                                    } else {
                                        MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                        console.error(successAprovacao);
                                    }
                                }, (error) => {
                                    MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                    console.error(error);
                                });
                            }
                            if(config.emailAutomatico == 'S') {
                                Dfs49000_TransmissaoNfe_Service.emailAutomaticoTransmissao(respConsulta[0].NOTAS_SUCCESS, config).then((successEmail) => {
                                    if(Object.keys(successEmail[0])[0] === 'SUCESSO_EMAIL' ) {
                                        MessageService.successNotificationByCode('SUCESSO_EMAIL_LOTE');
                                    } else if(Object.keys(successEmail[0])[0] === 'ERROS_EMAIL' ) {
                                        let erros   = successEmail[0].ERROS_EMAIL,
                                            dsErros = Dfs49000_TransmissaoNfe_Service.errosNotasTransmissao(erros);
                                        
                                        self.showPopupErros(dsErros);
                                    }
                                    else {
                                        MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                        console.error(successEmail);
                                    }
                                }, (error) => {
                                    MessageService.showMessageByCode('ERROR_APROVACAOEMAIL_LOTE');
                                    console.error(error);
                                });
                            }
                        }
                    }
                } else if(Object.keys(respConsulta[0])[0] === 'ERRORS') {
                    if(typeof respConsulta[0].ERRORS === 'string') {
                        ScreenService.showMessage(respConsulta[0].ERRORS, "ERROR");
                    } else {
                        MessageService.showMessageByCode("ERROR_CONSULTA_SITUACAONF");
                        console.error(respConsulta);
                    }
                } else {
                    MessageService.showMessageByCode("ERROR_CONSULTA_SITUACAONF");
                    console.error(respConsulta);
                }
            }, (error) => {
                MessageService.showMessageByCode("ERROR_CONSULTA_SITUACAONF");
                console.error(error);
            });
        });
    };
    
    this.gerarXml = function(widget) {
        let notasMarcadas = widget.getCheckedRows(),
            notas = notasMarcadas.filter(notafiscal => {
            return notafiscal.IDSTATUSNFE in {C:{}, A:{}} || notafiscal.IDTIPOEMISNFE == 4; //Libera xml para nfe cancelada, aceita ou EPEC.
        });
        
        if(notas.length > 0) {
            General.callRestEngine("df#/gerarXmlNfe", "DataSet", notas).then(response => {
                this.downloadArquivo(response.messages[0].message);
            });
        }
        else {
            MessageService.showMessageByCode('BLOCK_DOWN_XML');
        }
    };
    
    this.empresaUtlizaNFE = function(transmissaoNfeArgs){
        let widget =  transmissaoNfeArgs.owner.field.widget;
        let nmfilial = widget.getField("NMFILIAL");
        var cdempresa = this.getCurrentField('CDEMPRESA');
        General.callRestEngine("df_saida#/crud/paramempresa_saida", "FilterData", [{"name": "P_CDEMPRESA","operator": "=", "value": cdempresa}])
        .then((response) => {
            if(response.dataset.paramempresa_saida[0]){
                let utilizanfe = response.dataset.paramempresa_saida[0].IDEMPUTILIZANFE === 'S' ?  true : false;
                if (!utilizanfe){
                    ScreenService.showMessage("Empresa informada não utiliza nota fiscal eletrônica");
                    nmfilial.setValue("");
                    nmfilial.reload();
                }
            }
        });
    };
        
    this.fixZeroFill  = function (field) {
        var value = field.value();
        
        if (!value) {
            return;
        }
        
        //Remove pontos e vírgulas
        var newValue = value.replace('.', '').replace(',', '');
        
        //Realiza o zeroFill novamente
        var currentLength = newValue.toString().length;
        newValue = "0".repeat(field.maxlength - currentLength) + newValue;
        
        field.setValue(newValue);
    };
    
    this.checkInscricaoContaCorrente = function(field) {
        var inscricao = General.inscricaoFormat(field.value());
        
        if(inscricao.valid) {
            field.setValue(inscricao.value);
        } else {
            if(inscricao.tipo.cod == 'J') inscricao.tipo.msg = 'CNPJ'+" \""+inscricao.value+"\" inválido!";
            else if(inscricao.tipo.cod == 'F') inscricao.tipo.msg = 'CPF'+" \""+inscricao.value+"\" inválido!";
            else if(inscricao.tipo.cod == 'I') inscricao.tipo.msg = inscricao.tipo.desc+" \""+inscricao.value+"\" inválida!";
            
            if(inscricao.tipo.msg) ScreenService.showMessage(inscricao.tipo.msg, "warning");
        }
    };
    
    this.produtosOnEnter = function (widget) {
        var widgetTransmissao = widget.getParent();
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetTransmissao.currentRow.CDFILIAL
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": widgetTransmissao.currentRow.NRLANCTONF
            }
        ];
        widget.dataSourceFilter = filter;
        General.callRestEngine(widget.dataSource.name, "FilterData", widget.dataSourceFilter).then (response => {
            localStorageEngine.setDataSource(widget.dataSource.name, response.dataset.lancamento_nota_itens).then ( () => {
                widget.dataSource.reload();
            });
        });
    };
    
    this.produtosAfterMoveRow = function (widget) {
        var declaracaoWidget = widget.widgets.filter(widget => widget.name === 'declaracaoImportacao')[0];
        var adicoesWidget = widget.widgets.filter(widget => widget.name === 'adicoes')[0];
        this.declaracaoImportacaoOnEnter(declaracaoWidget).then (existeDeclaracao => {
            if (existeDeclaracao) {
                adicoesWidget.enabled = true;
            }
            else {
                adicoesWidget.enabled = false;
            }
            declaracaoWidget.activate();
        });
    };
    
    this.setCamposDeclaracaoImportacao = function (widget) {
        var fieldDSPAISADQUIRE =  widget.getField("DSPAISADQUIRE");
        var fieldDSPAISLOCEMBIMP = widget.getField("DSPAISLOCEMBIMP");
        var fieldNMESTADOADQUIRE = widget.getField("NMESTADOADQUIRE");
        var fieldNMUFLOCEMBIMP = widget.getField("NMUFLOCEMBIMP");
        var value;
        
        fieldNMESTADOADQUIRE.readOnly = false;
        fieldNMUFLOCEMBIMP.readOnly = false;
        
        fieldDSPAISADQUIRE.reload().then ( () => {
            value = fieldDSPAISADQUIRE.dataSource.data.filter(data => data.CDPAIS === widget.currentRow.CDPAISADQUIRE)[0];
            if (!value) {
                fieldDSPAISADQUIRE.setValue("");
                fieldNMESTADOADQUIRE.setValue("");
                fieldNMESTADOADQUIRE.readOnly = true;
                return;
            }
            fieldDSPAISADQUIRE.setValue(value.NMPAIS);
            fieldNMESTADOADQUIRE.dataSource.reload().then ( () => {
                value = fieldNMESTADOADQUIRE.dataSource.data.filter(data => data.SGESTADO === widget.currentRow.SGESTADOADQUIRE)[0];
                if (!value) {
                    fieldNMESTADOADQUIRE.setValue("");
                    return;
                }
                fieldNMESTADOADQUIRE.setValue(value.NMESTADO);
            });
        });
        
        fieldDSPAISLOCEMBIMP.reload().then ( () => {
            value = fieldDSPAISLOCEMBIMP.dataSource.data.filter(data => data.CDPAIS === widget.currentRow.CDPAISLOCEMBIMP)[0];
            if (!value) {
                fieldDSPAISLOCEMBIMP.setValue("");
                fieldNMUFLOCEMBIMP.setValue("");
                fieldNMUFLOCEMBIMP.readOnly = true;
                return;
            }
            fieldDSPAISLOCEMBIMP.setValue(value.NMPAIS);
            fieldNMUFLOCEMBIMP.dataSource.reload().then ( () => {
                value = fieldNMUFLOCEMBIMP.dataSource.data.filter(data => data.SGESTADO === widget.currentRow.SGUFLOCEMBIMP)[0];
                if (!value) {
                   fieldNMUFLOCEMBIMP.setValue("");
                   return;
                }
                fieldNMUFLOCEMBIMP.setValue(value.NMESTADO);
            });
        });
    };
    
    this.declaracaoImportacaoLoadData = function (widget) {
        if (widget.dataSource.data[0]) {
            delete widget.dataSource.data[0];
        }
        for (let key in widget.currentRow) {
            widget.currentRow[key] = "";
        }
        var promise = ZHPromise.defer(); 
        General.callRestEngine(widget.dataSource.name, "FilterData", widget.dataSourceFilter).then (response => {
            let dataset = response.dataset.decimpnfe[0];
            if (dataset) {
                localStorageEngine.setDataSource(widget.dataSource.name, dataset).then ( () => {
                    widget.dataSource.reload();
                    widget.currentRow = dataset;
                    widget.currentRow.__is_new = false;
                    this.setCamposDeclaracaoImportacao(widget);
                    promise.resolve(true);
                });
            }
            else {
                 localStorageEngine.remove(widget.dataSource.name).then (() => {
                     widget.getField("NMESTADOADQUIRE").readOnly = true;
                     widget.getField("NMUFLOCEMBIMP").readOnly = true;
                     widget.currentRow.__is_new = true;
                     widget.dataSourceFilter.forEach (filter => {
                         widget.currentRow[filter.name] = filter.value;
                     });
                     promise.resolve(false);
                 });
            }
        });
        return promise;
    };
    
    this.declaracaoImportacaoOnEnter = function (widget) {
        let idStatusNFErow = widget.getParent().getParent().currentRow.IDSTATUSNFE;
        var widgetProdutos = widget.getParent();
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetProdutos.currentRow.CDFILIAL
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": widgetProdutos.currentRow.NRLANCTONF
            },
            {
                "name": "NRSEQITPROD",
                "operator": "=",
                "value": widgetProdutos.currentRow.NRSEQITPROD
            }
        ];
        widget.dataSourceFilter = filter;
        
        if(idStatusNFErow == 'P' || idStatusNFErow == 'R')  widget.getAction('editar').isVisible = true;
        else                                                widget.getAction('editar').isVisible = false;
        
        return this.declaracaoImportacaoLoadData(widget);
    };
    
    this.paisOnChange = function (field) {
        var widget = field.widget;
        var pais = field.value();
        switch (field.name) {
            case "DSPAISADQUIRE": 
                widget.currentRow.SGESTADOADQUIRE = "";
                widget.currentRow.NMESTADOADQUIRE = "";
                if (pais) {
                    widget.getField("NMESTADOADQUIRE").readOnly = false;
                }
                else {
                    widget.getField("NMESTADOADQUIRE").readOnly = true;
                }
                break;
            case "DSPAISLOCEMBIMP": 
                widget.currentRow.SGUFLOCEMBIMP = "";
                widget.currentRow.NMUFLOCEMBIMP = "";
                if (pais) {
                    widget.getField("NMUFLOCEMBIMP").readOnly = false;
                }
                else {
                    widget.getField("NMUFLOCEMBIMP").readOnly = true;
                }
                break;
        }
    };
    
    this.saveDeclaracaoImportacao = function (widget) {
        if (!widget.isValid()) {
            return;
        }
        let isNewDI = widget.currentRow.__is_new ? true : false;
        var adicoesWidget = widget.getParent().widgets.filter(widget => widget.name === 'adicoes')[0];
        General.callRestEngine(widget.dataSource.name+'/save', "Row", widget.currentRow).then( () => {
            this.declaracaoImportacaoLoadData(widget);
            this.setCamposDeclaracaoImportacao(widget);
            widget.list();
            adicoesWidget.enabled = true;
            if(isNewDI) {
                adicoesWidget.activate().then( () => {
                    adicoesWidget.reload().then( () => {
                        adicoesWidget.dataSource.data = [];
                        this.adicoesNewRow(adicoesWidget, true);
                    });
                });
            }
        });
    };
    
    this.cancelarDeclaracaoImportacao = function (widget) {
        widget.list();
        this.declaracaoImportacaoLoadData(widget);
    };
    
    this.adicoesOnEnter = function (widget) {
        let idStatusNFErow = widget.getParent().getParent().currentRow.IDSTATUSNFE;
        var widgetProdutos = widget.getParent();
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetProdutos.currentRow.CDFILIAL
            },
            {
                "name": "NRLANCTONF",
                "operator": "=",
                "value": widgetProdutos.currentRow.NRLANCTONF
            },
            {
                "name": "NRSEQITPROD",
                "operator": "=",
                "value": widgetProdutos.currentRow.NRSEQITPROD
            }
        ];
        widget.dataSourceFilter = filter;
        
        if(idStatusNFErow == 'P' || idStatusNFErow == 'R') {
            widget.getAction('editar').isVisible = true;
            widget.getAction('novoServico').isVisible = true;
        }
        else {
            widget.getAction('editar').isVisible = false;
            widget.getAction('novoServico').isVisible = false;
        }
        
        General.callRestEngine(widget.dataSource.name, "FilterData", widget.dataSourceFilter).then (response => {
            let idStatusNFErow = widget.getParent().getParent().currentRow.IDSTATUSNFE;
            
            if(response.dataset.adicdecimpnfe.length <= 1 )          widget.getAction('excluir').isVisible = false;
            else if(idStatusNFErow == 'P' || idStatusNFErow == 'R')  widget.getAction('excluir').isVisible = true;
            else                                                     widget.getAction('excluir').isVisible = false;
            
            localStorageEngine.setDataSource(widget.dataSource.name, response.dataset.adicdecimpnfe).then ( () => {
                widget.dataSource.reload();
            });
        });
    };
    
    this.adicoesNewRow = function (widget, ocultaActCancel) {
        let NRSEQADICDEC = 0;
        
        widget.newRow();
        for (let field in widget.dataSourceFilter) {
            widget.currentRow[widget.dataSourceFilter[field].name] = widget.dataSourceFilter[field].value;
        }
        for (let index in widget.dataSource.data) {
            if (widget.dataSource.data[index].NRSEQADICDEC > NRSEQADICDEC) {
                NRSEQADICDEC = widget.dataSource.data[index].NRSEQADICDEC;
            }
        }
        NRSEQADICDEC++;
        widget.currentRow.NRSEQADICDEC = NRSEQADICDEC;
        
        if(ocultaActCancel) widget.getAction('cancelar').isVisible = false;
        else                widget.getAction('cancelar').isVisible = true;
    };
    
    this.saveAdicoes = function (widget) {
        if (!widget.isValid()) {
            return;
        }
        General.callRestEngine(widget.dataSource.name+'/save', "Row", widget.currentRow).then( () => {
            this.adicoesOnEnter(widget);
            ScreenService.successNotification("Registro salvo com sucesso!");
            widget.view();
            ScreenService.goBack();
        });
    };
    
    this.cancelarAdicoes = function (widget) {
        this.adicoesOnEnter(widget);
        widget.list();
        ScreenService.goBack();
    };
    
    this.excluirAdicoes = function (widget) {
        ScreenService.confirmMessage('Deseja excluir o registro?', 'question', function (success) {
            General.callRestEngine(widget.dataSource.name+'/remove', "Row", widget.currentRow).then( () => {
                self.adicoesOnEnter(widget);
                ScreenService.successNotification("Registro excluído com sucesso!");
                widget.view();
                ScreenService.goBack();
            });
        }, requestError());
    };
    
    this.codExportMaskOnBlur = (field) => {
        field.setValue(field.value().replace(/[^\d]+/g, ""));
    };
    
    this.entSaidFilterOnChange = (wid) => {
        wid.getField('DTENTRSAID').label = wid.currentRow.IDENTRSAIDOP == 'E' ? 'Data Entrada' : 'Data Saída';
        wid.reload();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Dfs49000_TransmissaoNfe_Controller", dfs49000_TransmissaoNfe_Controller);
});